#tab_package_list thead th {
    text-align: left;
    vertical-align: middle;
    white-space: nowrap;
}

#tab_package_list tbody td {
    text-align: left;
    vertical-align: middle;
    word-wrap: break-word;
}

#tab_package_list tbody td:nth-child(1) {
    white-space: nowrap;
}

#tab_package_list tbody td:nth-child(2) {
    white-space: nowrap;
}

#tab_package_list tbody td:nth-child(3) {
    white-space: nowrap;
}

#tab_package_list tbody td:nth-child(5) {
    white-space: nowrap;
}

#tab_package_list tbody td:nth-child(6) {
    white-space: nowrap;
}

.dotted_underline {
    text-decoration: underline;
    text-decoration-style: dotted;
}

.copy_to_clipboard:hover {
    cursor: pointer;
}
