"""An app that helps keep track of installed packages
and outstanding updates for Alliance Auth.
"""

# pylint: disable=invalid-name
default_app_config = "package_monitor.apps.PackageMonitorConfig"

__version__ = "1.15.0"
__title__ = "Package Monitor"
